class Shield : public AR_Sprite
 {
  protected:

   Bullet **bullets; // Array of bullets (for collision)
   int health,max_health;
   int recharge_step,recharge_count;
   bool down,recharge_enable;

  public:

   Shield(string filename, AR_Display *display, AR_Sprite *p = NULL, int collision_mask_type = AR::STATIC_COLLISION_MASK) : AR_Sprite(filename,display,0,0,p,collision_mask_type)
    {
	 reset();
	}

   ~Shield() { }

   void run()
    {
     if(!enable) return;

     AR_Sprite::run();
     if(action==AR::NONE) current_frame = 1;
     if(action==AR::HIT)
      {
       current_frame = 3;
       alpha  = 250;
       action = AR::FADE;
      }
     if(action==AR::FADE)
      {
       alpha-=10;
       if(alpha==0) action = AR::NONE;
      }
     if(action==AR::DOWN)
      {
	   scale_x-=0.018; scale_y-=0.018; alpha-=4;
	   if(alpha==0) { enable = false; down = true; action = AR::NONE; scale_x = scale_y = 1; }
	  }

	 // Recharge shield
	 if(recharge_enable && health >0)
	  {
       if(recharge_count > recharge_step) recharge_count = recharge_step;
	   recharge_count--;
	   if(recharge_count == 0) { recharge_count = recharge_step; health++; }
	   if(health > max_health) health = max_health;
      }
	}

   void draw()
    {
	 AR_Sprite::draw();
	 if(action==AR::FADE || action==AR::DOWN)
	  { current_frame = 2; AR_Sprite::draw(); current_frame = 3; }
	}

   void hit(Bullet *b)
    {
	 action = AR::HIT;
	 health -= b->getDamage();
	 angle = AR_Util::degrees(atan2(b->getY()-y,b->getX()-x));
	 if(health <= 0) { health = 0; alpha = 252; current_frame = 2; action = AR::DOWN; recharge_enable = false; }
	}

   void reset()
    {
	 // Reset variables to default
	 AR_Sprite::reset();
	 setCenterX(width>>1); setCenterY(height>>1);
	 health = max_health = 0;
	 enable = false;
	 down   = false;
	 current_frame  = 1;
	 recharge_step   = display->getMaxFPS() * 60;
	 recharge_count  = recharge_step;
	 recharge_enable = false;
	}

   void setScrollXYP(float *scroll_x, float * scroll_y)
    {
	 setScrollXP(scroll_x); setScrollYP(scroll_y);
	}

   bool getDown()        { return down;          }

   int getRechargeStep() { return recharge_step; }

   void setHealth(int health)                   { this->health = health;                   }
   
   void setMaxHealth(int max_health)            { this->max_health = max_health;           }

   void setRechargeEnable(bool recharge_enable) { this->recharge_enable = recharge_enable; }

   void setRechargeStep(int recharge_step)      { this->recharge_step = recharge_step;     }
   
   int getHealth()    { return health;     }
   
   int getMaxHealth() { return max_health; }

 };
